@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -101,7 +101,7 @@ USA.
   (ASSIGN (REGISTER (? target))
 	  (CONS-POINTER (REGISTER (? type)) (REGISTER (? datum))))
   (let ((temp (standard-move-to-temporary! type)))
-    (LAP (ROR W ,temp (&U ,scheme-type-width))
+    (LAP (ROR W ,temp (& ,scheme-type-width))
 	 (OR W ,(standard-move-to-target! datum target) ,temp))))
 
 (define-rule statement
@@ -317,7 +317,7 @@ USA.
     (LAP (MOV B ,target ,source))))
 
 (define (char->signed-8-bit-immediate character)
-  (let ((ascii (char->ascii character)))
+  (let ((ascii (char->integer character)))
     (if (< ascii 128) ascii (- ascii 256))))
 
 ;;;; Utilities specific to rules1
