@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -53,7 +53,7 @@ USA.
 (define-integrable (set-package/children! package children)
   (%record-set! package 2 children))
 
-(define-integrable (package/%name package)
+(define-integrable (package/name package)
   (%record-ref package 3))
 
 (define-integrable (package/environment package)
@@ -65,43 +65,34 @@ USA.
 (define (package-name? object)
   (list-of-type? object symbol?))
 
+(define (package-name=? name1 name2)
+  (or (and (null? name1) (null? name2))
+      (and (pair? name1)
+	   (pair? name2)
+	   (eq? (car name1) (car name2))
+	   (package-name=? (cdr name1) (cdr name2)))))
+
 (define (package/reference package name)
   (lexical-reference (package/environment package) name))
 
 (define (finalize-package-record-type!)
   (let ((rtd
-	 (make-record-type "package" '(PARENT CHILDREN %NAME ENVIRONMENT))))
-    (let ((tag (record-type-dispatch-tag rtd)))
-      (set! package-tag tag)
-      (let loop ((package system-global-package))
-	(%record-set! package 0 tag)
-	(for-each loop (package/children package))))
-    (set-record-type-unparser-method! rtd
-      (standard-unparser-method 'PACKAGE
-	(lambda (package port)
-	  (write-char #\space port)
-	  (write (package/name package) port))))))
+	 (make-record-type "package" '(parent children name environment))))
+    (set! package-tag rtd)
+    (for-each (lambda (p) (%record-set! p 0 rtd)) *packages*)
+    (define-print-method (record-predicate rtd)
+      (standard-print-method 'package
+	(lambda (package)
+	  (list (package/name package)))))))
 
-(define (package/child package name)
-  (let loop ((children (package/children package)))
-    (and (pair? children)
-	 (if (eq? name (package/%name (car children)))
-	     (car children)
-	     (loop (cdr children))))))
-
-(define (package/name package)
-  (let loop ((package package) (result '()))
-    (if (package/parent package)
-	(loop (package/parent package) (cons (package/%name package) result))
-	result)))
-
 (define (name->package name)
-  (let loop ((path name) (package system-global-package))
-    (if (pair? path)
-	(let ((child (package/child package (car path))))
-	  (and child
-	       (loop (cdr path) child)))
-	package)))
+  (find-package name #f))
+
+(define (all-packages)
+  (let loop ((packages *packages*))
+    (if (pair? packages)
+	(cons (car packages) (loop (cdr packages)))
+	'())))
 
 (define (environment->package environment)
   (and (interpreter-environment? environment)
@@ -115,32 +106,27 @@ USA.
 	      package))))
 
 (define-integrable package-name-tag
-  ((ucode-primitive string->symbol) "#[(package)package-name-tag]"))
+  '|#[(package)package-name-tag]|)
 
 (define (find-package name #!optional error?)
-  (let loop ((path name) (package system-global-package))
-    (if (pair? path)
-	(loop (cdr path)
-	      (let ((child (package/child package (car path))))
-		(if (and (not child) error?)
-		    (error "Unable to find package:"
-			   (list-difference name (cdr path))))
-		child))
-	package)))
+  (let package-loop ((packages *packages*))
+    (if (pair? packages)
+	(if (package-name=? name (package/name (car packages)))
+	    (car packages)
+	    (package-loop (cdr packages)))
+	(begin
+	  (if error? (error "Unable to find package:" name))
+	  #f))))
 
-(define (list-difference list tail)
-  (let loop ((list list))
-    (if (eq? list tail)
-	'()
-	(cons (car list) (loop (cdr list))))))
+(define (name-append name package)
+  (let loop ((names (package/name package)))
+    (if (pair? names)
+	(cons (car names) (loop (cdr names)))
+	(cons name '()))))
 
 (define (package/add-child! package name environment #!optional force?)
-  (let ((child (package/child package name))
-	(finish
-	 (lambda (child)
-	   (if (not (interpreter-environment->package environment))
-	       (local-assignment environment package-name-tag child))
-	   child)))
+  (let* ((real-name (name-append name package))
+	 (child (find-package real-name #f)))
     (if child
 	(begin
 	  (if (not (if (default-object? force?)
@@ -149,20 +135,17 @@ USA.
 	      (error "Package already has child of given name:" package name))
 	  (set-package/environment! child environment)
 	  (set-package/children! child '())
-	  (finish child))
-	(let ((child (make-package package name environment)))
-	  (set-package/children! package
-				 (cons child (package/children package)))
-	  (finish child)))))
+	  (if (not (interpreter-environment->package environment))
+	      (local-assignment environment package-name-tag child))
+	  child)
+	(package/create real-name package environment))))
 
-(define system-global-package)
+(define *packages* '())
 (define *allow-package-redefinition?* #f)
 
 (define (initialize-package!)
-  (set! system-global-package (make-package #f #f system-global-environment))
-  (local-assignment system-global-environment
-		    package-name-tag
-		    system-global-package))
+  (set! *packages* '())
+  (package/create '() #f system-global-environment))
 
 (define (load-package-set filename #!optional options)
   (let ((pathname (merge-pathnames filename))
@@ -170,7 +153,7 @@ USA.
     (let ((dir (directory-pathname pathname))
 	  (pkg (package-set-pathname pathname os-type))
 	  (options
-	   (cons (cons 'OS-TYPE os-type)
+	   (cons (cons 'os-type os-type)
 		 (if (default-object? options) '() options))))
       (with-working-directory-pathname dir
 	(lambda ()
@@ -179,10 +162,10 @@ USA.
 		(error "Malformed package-description file:" pkg))
 	    (construct-packages-from-file file)
 	    (let ((alternate-loader
-		   (lookup-option 'ALTERNATE-PACKAGE-LOADER options))
+		   (lookup-option 'alternate-package-loader options))
 		  (load-component
 		   (lambda (name environment)
-		     (load name environment 'DEFAULT #t))))
+		     (load name environment 'default #t))))
 	      (if alternate-loader
 		  (alternate-loader load-component options)
 		  (begin
@@ -197,18 +180,22 @@ USA.
 (define system-loader/enable-query? #f)
 
 (define (package-set-pathname pathname #!optional os-type)
-  (pathname-new-type
-   (pathname-new-name pathname
-		      (string-append (pathname-name pathname)
-				     "-"
-				     (case (if (default-object? os-type)
-					       microcode-id/operating-system
-					       os-type)
-				       ((NT) "w32")
-				       ((OS/2) "os2")
-				       ((UNIX) "unx")
-				       (else "unk"))))
-   "pkd"))
+  (let ((p (->pathname pathname)))
+    (pathname-new-type
+     (pathname-new-name p
+			(string-append
+			 (or (pathname-name p)
+			     ;; Interpret dirname/ as dirname/dirname-OS.pkd.
+			     (let ((dir (pathname-directory p)))
+			       (if (pair? dir)
+				   (let ((name (last dir)))
+				     (if (string? name)
+					 name
+					 ""))
+				   "")))
+			 "-"
+			 (microcode-id/operating-system-suffix os-type)))
+     "pkd")))
 
 (define-integrable (make-package-file tag version descriptions loads)
   (vector tag version descriptions loads))
@@ -241,7 +228,7 @@ USA.
 (define (package-file? object)
   (and (vector? object)
        (fix:= (vector-length object) 4)
-       (eq? (package-file/tag object) 'PACKAGE-DESCRIPTIONS)
+       (eq? (package-file/tag object) 'package-descriptions)
        (and (index-fixnum? (package-file/version object))
 	    (fix:= (package-file/version object) 2))
        (vector-of-type? (package-file/descriptions object)
@@ -268,6 +255,12 @@ USA.
 	      (and (symbol? (vector-ref object 0))
 		   (package-name? (vector-ref object 1))
 		   (symbol? (vector-ref object 2))))
+	     ((fix:= (vector-length object) 4)
+	      (and (symbol? (vector-ref object 0))
+		   (package-name? (vector-ref object 1))
+		   (symbol? (vector-ref object 2))
+		   (or (eq? #f (vector-ref object 3))
+		       (eq? 'deprecated (vector-ref object 3)))))
 	     (else #f))))
 
 (define (load-description? object)
@@ -281,15 +274,16 @@ USA.
 		    (vector-of-type? (cdr file-case)
 		      (lambda (clause)
 			(and (pair? clause)
-			     (or (eq? (car clause) 'ELSE)
+			     (or (eq? (car clause) 'else)
 				 (vector-of-type? (car clause) symbol?))
 			     (vector-of-type? (cdr clause) string?)))))
 	       (vector-of-type? file-case string?))))
        (vector? (load-description/initializations object))
        (vector? (load-description/finalizations object))))
 
-;; CONSTRUCT-PACKAGES-FROM-FILE is called from the cold load and must
-;; only use procedures that are inline-coded by the compiler.
+;; CONSTRUCT-PACKAGES-FROM-FILE is called from the cold load before
+;; the runtime system is loaded.  Thus it must only call procedures
+;; that are defined in this file.
 
 (define (construct-packages-from-file file)
   (let ((descriptions (package-file/descriptions file))
@@ -297,15 +291,19 @@ USA.
 	 (lambda (name)
 	   (or (null? name)
 	       (and (pair? name)
-		    (eq? (car name) 'PACKAGE)
+		    (eq? (car name) 'package)
 		    (null? (cdr name)))))))
     (let ((n (vector-length descriptions)))
       (do ((i 0 (fix:+ i 1)))
 	  ((fix:= i n))
 	(let ((description (vector-ref descriptions i)))
 	  (let ((name (package-description/name description)))
-	    (if (not (skip-package? name))
-		(construct-normal-package-from-description description)))))
+	    (if (and (not (skip-package? name))
+		     (not (package-description/extension? description))
+		     ;; If there is an existing package, treat this as
+		     ;; though an extension.
+		     (not (find-package name #f)))
+		(create-package-from-description description)))))
       (do ((i 0 (fix:+ i 1)))
 	  ((fix:= i n))
 	(let ((description (vector-ref descriptions i)))
@@ -313,29 +311,29 @@ USA.
 	    (if (not (skip-package? name))
 		(create-links-from-description description))))))))
 
-(define (construct-normal-package-from-description description)
-  (let ((name (package-description/name description))
-	(extension? (package-description/extension? description))
-	(environment
-	 (extend-package-environment
-	  (let ((ancestors (package-description/ancestors description)))
-	    (if (pair? ancestors)
-		(package/environment (find-package (car ancestors)))
-		null-environment))
-	  (cons (package-description/internal-names description)
-		(lambda (name) name))
-	  (cons (package-description/exports description)
-		(lambda (binding) (vector-ref binding 0)))
-	  (cons (package-description/imports description)
-		(lambda (binding) (vector-ref binding 0))))))
-    (let loop ((path name) (package system-global-package))
-      (if (pair? (cdr path))
-	  (loop (cdr path)
-		(or (package/child package (car path))
-		    (error "Unable to find package:"
-			   (list-difference name (cdr path)))))
-	  (if (not (and extension? (package/child package (car path))))
-	      (package/add-child! package (car path) environment))))))
+(define (create-package-from-description description)
+  (let* ((parent (let ((ancestors (package-description/ancestors description)))
+		   (if (pair? ancestors)
+		       (find-package (car ancestors))
+		       #f)))
+	 (environment
+	  (extend-package-environment
+	   (if parent (package/environment parent) null-environment)
+	   (cons (package-description/internal-names description)
+		 (lambda (name) name))
+	   (cons (package-description/exports description)
+		 (lambda (binding) (vector-ref binding 0)))
+	   (cons (package-description/imports description)
+		 (lambda (binding) (vector-ref binding 0))))))
+    (package/create (package-description/name description) parent environment)))
+
+(define (package/create name parent environment)
+  (let ((new (make-package parent name environment)))
+    (local-assignment environment package-name-tag new)
+    (if parent
+	(set-package/children! parent (cons new (package/children parent))))
+    (set! *packages* (cons new *packages*))
+    new))
 
 (define (create-links-from-description description)
   (let ((environment
@@ -346,7 +344,7 @@ USA.
 	    ((fix:= i n))
 	  (let ((binding (vector-ref bindings i)))
 	    (link-variables (find-package-environment (vector-ref binding 1))
-			    (if (fix:= (vector-length binding) 3)
+			    (if (fix:>= (vector-length binding) 3)
 				(vector-ref binding 2)
 				(vector-ref binding 0))
 			    environment
@@ -359,7 +357,7 @@ USA.
 	    (let ((source-environment
 		   (find-package-environment (vector-ref binding 1)))
 		  (source-name
-		   (if (fix:= (vector-length binding) 3)
+		   (if (fix:>= (vector-length binding) 3)
 		       (vector-ref binding 2)
 		       (vector-ref binding 0))))
 	      (guarantee-binding source-environment source-name)
@@ -395,7 +393,7 @@ USA.
 	     ((ucode-primitive vector-cons)
 	      n
 	      (make-unmapped-unassigned-reference-trap))))
-	(vector-set! vn 0 'DUMMY-PROCEDURE)
+	(vector-set! vn 0 'dummy-procedure)
 	(do ((names names (cdr names))
 	     (j 1 (fix:+ j 1)))
 	    ((not (pair? names)))
@@ -411,7 +409,7 @@ USA.
 (define null-environment
   ((ucode-primitive object-set-type)
    ((ucode-primitive object-type) #f)
-   (fix:xor ((ucode-primitive object-datum) #F) 1)))
+   (fix:xor ((ucode-primitive object-datum) #f) 1)))
 
 (define (find-package-environment name)
   (package/environment (find-package name)))
@@ -471,7 +469,7 @@ USA.
 			  ((fix:= i n))
 			(let ((clause (vector-ref clauses i)))
 			  (if (let ((keys (car clause)))
-				(or (eq? keys 'ELSE)
+				(or (eq? keys 'else)
 				    (let ((n (vector-length keys)))
 				      (let loop ((i 0))
 					(and (fix:< i n)
