@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -31,7 +31,7 @@ USA.
 	 (integrate-external "object"))
 
 (define (variable/make&bind! block name)
-  (guarantee-symbol name 'variable/make&bind!)
+  (guarantee symbol? name 'variable/make&bind!)
   (or (%block/lookup-name block name)
       (%variable/make&bind! block name)))
 
@@ -42,7 +42,7 @@ USA.
     variable))
 
 (define (block/lookup-name block name intern?)
-  (guarantee-symbol name 'block/lookup-name)
+  (guarantee symbol? name 'block/lookup-name)
   (let search ((block block))
     (or (%block/lookup-name block name)
 	(if (block/parent block)
@@ -50,12 +50,12 @@ USA.
 	    (and intern? (%variable/make&bind! block name))))))
 
 (define (%block/lookup-name block name)
-  (find-matching-item (block/bound-variables block)
-		      (lambda (variable)
-			(eq? (variable/name variable) name))))
+  (find (lambda (variable)
+	  (eq? (variable/name variable) name))
+	(block/bound-variables block)))
 
 (define (block/limited-lookup block name limit)
-  (guarantee-symbol name 'block/limited-lookup)
+  (guarantee symbol? name 'block/limited-lookup)
   (let search ((block block))
     (and (not (eq? block limit))
 	 (or (%block/lookup-name block name)
